package com.example.lifecyclelogger;

import android.content.Context;
import android.os.Bundle;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.room.Room;

import com.example.lifecyclelogger.AppDatabase;
import com.example.lifecyclelogger.LifecycleEvent;
import com.example.lifecyclelogger.LifecycleEventDao;

import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class MainActivity extends AppCompatActivity {

    private ListView listView;
    private Button clearHistoryButton;
    private TextView textViewTimestamp;
    private ArrayAdapter<String> adapter;
    private List<String> lifecycleEvents = new ArrayList<>();
    private String logFileName = "lifecycle_log.txt";

    private AppDatabase database;
    private LifecycleEventDao eventDao;
    private ExecutorService executorService = Executors.newSingleThreadExecutor();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        listView = findViewById(R.id.listView);
        clearHistoryButton = findViewById(R.id.button_clear_history);
        //textViewTimestamp = findViewById(R.id.textView_timestamp);

        // Инициализация базы данных Room
        database = Room.databaseBuilder(getApplicationContext(), AppDatabase.class, "lifecycle-events-db")
                .fallbackToDestructiveMigration()
                .build();

        eventDao = database.eventDao();

        clearHistoryButton.setOnClickListener(v -> {
            clearHistory();
            logLifecycleEvent("Clear History Button Clicked");
        });

        adapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, lifecycleEvents);
        listView.setAdapter(adapter);

        loadHistory();
        logLifecycleEvent("onCreate");
    }

    @Override
    protected void onStart() {
        super.onStart();
        logLifecycleEvent("onStart");
    }

    @Override
    protected void onResume() {
        super.onResume();
        logLifecycleEvent("onResume");
    }

    @Override
    protected void onPause() {
        super.onPause();
        logLifecycleEvent("onPause");
    }

    @Override
    protected void onStop() {
        super.onStop();
        logLifecycleEvent("onStop");
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        logLifecycleEvent("onDestroy");
        executorService.shutdown();
    }

    private void logLifecycleEvent(String event) {
        long timestamp = System.currentTimeMillis();
        String logEntry = event + " - " + formatTimestamp(timestamp);
        lifecycleEvents.add(logEntry);
        adapter.notifyDataSetChanged();
        saveToLogFile(logEntry);

        // Сохранение в Room
        executorService.execute(() -> {
            eventDao.insertEvent(new LifecycleEvent(event, timestamp));
        });
    }

    private String formatTimestamp(long timestamp) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sdf.format(new Date(timestamp));
    }

    private void saveToLogFile(String logEntry) {
        try (FileOutputStream fos = openFileOutput(logFileName, Context.MODE_APPEND)) {
            fos.write((logEntry + "\n").getBytes());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void loadHistory() {
        lifecycleEvents.clear();
        executorService.execute(() -> {
            List<LifecycleEvent> events = eventDao.getAllEvents();
            for (LifecycleEvent event : events) {
                lifecycleEvents.add(event.getEventName() + " - " + formatTimestamp(event.getTimestamp()));
            }
            runOnUiThread(() -> adapter.notifyDataSetChanged());
        });
    }

    private void clearHistory() {
        executorService.execute(() -> {
            eventDao.deleteAllEvents();
            runOnUiThread(() -> {
                lifecycleEvents.clear();
                adapter.notifyDataSetChanged();
            });
        });
    }
}
